#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2020, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# CBOR Content for CMS
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc8769.txt
#

from pyasn1.type import univ

id_ct_cbor = univ.ObjectIdentifier("1.2.840.113549.1.9.16.1.44")


id_ct_cborSequence = univ.ObjectIdentifier("1.2.840.113549.1.9.16.1.45")
